==> sghi_core-0.2.38.tar.gz: test/communities/unit_tests/core/community_utils_test.dart <==
import 'dart:io';

import 'package:flutter_test/flutter_test.dart';
import 'package:intl/intl.dart';
import 'package:sghi_core/communities/core/community_utils.dart';
import 'package:sghi_core/communities/core/entities/event_types.dart';
import 'package:sghi_core/communities/core/enums.dart';
import 'package:sghi_core/communities/models/message.dart';
import 'package:sghi_core/communities/models/room.dart';
import 'package:sghi_core/communities/models/sync_response.dart';

import '../../mocks.dart';
import '../../response_mocks.dart';

void main() {
  group('utils', () {
    test('should convert mime types', () async {
      expect(() => convertMimeTypes(File('example.png'), null),
          throwsA(isA<String>()));

      expect(convertMimeTypes(File('example.png'), 'image/png'),
          equals('image/png'));

      expect(convertMimeTypes(File('example.heic'), 'image/png'),
          equals('image/heic'));
    });

    test('should pick an image', () async {
      final File? file = await selectPhoto(picker: ImagePickerMock());

      expect(file, isNotNull);
      expect(file, isA<File>());
      expect(file?.path, isNotNull);
    });

    test('should format a username', () async {
      expect(formatUsername(''), '');
      expect(formatUsername('abiud'), 'abiud');
      expect(formatUsername('abiud:chat.matrix:org'), 'abiud');
      expect(formatUsername('@abiud:chat.matrix:org'), '@abiud');
    });

    test('formatTimestamp formats timestamp correctly', () {
      const int timestamp = 1683888728201;
      final String formattedTimestamp = formatTimestamp(timestamp: timestamp);

      final String expectedFormattedTimestamp =
          '${DateFormat('d MMM, yy').format(DateTime.fromMillisecondsSinceEpoch(timestamp))} at ${DateFormat('jm').format(DateTime.fromMillisecondsSinceEpoch(timestamp))}';

      expect(formattedTimestamp, equals(expectedFormattedTimestamp));
    });

    test('formatTimestamp returns empty string when timestamp is null', () {
      final String formattedTimestamp = formatTimestamp(timestamp: null);

      expect(formattedTimestamp, equals(''));
    });

    test('should organize events by< timestamp a timestamp', () async {
      expect(organizeEventTimeline(<Message>[]), <Message>[]);

      expect(
        organizeEventTimeline(<Message>[
          Message.initial(),
          Message.initial(),
        ]),
        <Message>[
          Message.initial(),
          Message.initial(),
        ],
      );

      expect(
        organizeEventTimeline(<Message>[
          Message.initial().copyWith.call(timeStamp: 1683888728201),
          Message.initial().copyWith.call(timeStamp: 1683888728202),
        ]),
        <Message>[
          Message.initial().copyWith.call(timeStamp: 1683888728202),
          Message.initial().copyWith.call(timeStamp: 1683888728201),
        ],
      );
    });

    test('should return a valid fibonacci sequence', () {
      expect(fibonacci(0), <int>[0]);

      expect(fibonacci(1), <int>[0, 1]);

      expect(fibonacci(2), <int>[0, 1, 1]);

      expect(fibonacci(3), <int>[0, 1, 1, 2]);
    });

    test('formatMessageTimestamp returns correct string', () {
      expect(formatMessageTimestamp(null), '');
      expect(
        formatMessageTimestamp(DateTime.now().millisecondsSinceEpoch),
        'Just now',
      );
      expect(
        formatMessageTimestamp(
          DateTime.now()
              .subtract(const Duration(seconds: 30))
              .millisecondsSinceEpoch,
        ),
        'A few seconds ago',
      );
      expect(
        formatMessageTimestamp(
          DateTime.now()
              .subtract(const Duration(minutes: 1))
              .millisecondsSinceEpoch,
        ),
        '1 minute ago',
      );
      expect(
        formatMessageTimestamp(
          DateTime.now()
              .subtract(const Duration(minutes: 5))
              .millisecondsSinceEpoch,
        ),
        '5 minutes ago',
      );
      expect(
        formatMessageTimestamp(
          DateTime.now()
              .subtract(const Duration(hours: 1))
              .millisecondsSinceEpoch,
        ),
        'Today at ${DateFormat('h:mm a').format(DateTime.now().subtract(const Duration(hours: 1)))}',
      );
      expect(
        formatMessageTimestamp(
          DateTime.now()
              .subtract(const Duration(hours: 23))
              .millisecondsSinceEpoch,
        ),
        'Yesterday at ${DateFormat('h:mm a').format(DateTime.now().subtract(const Duration(hours: 23)))}',
      );
      expect(
        formatMessageTimestamp(
          DateTime.now()
              .subtract(const Duration(days: 3))
              .millisecondsSinceEpoch,
        ),
        DateFormat('EEEE')
            .format(DateTime.now().subtract(const Duration(days: 3))),
      );
      expect(
        formatMessageTimestamp(
          DateTime.now()
              .subtract(const Duration(days: 30))
              .millisecondsSinceEpoch,
        ),
        DateFormat('d MMMM')
            .format(DateTime.now().subtract(const Duration(days: 30))),
      );
      expect(
        formatMessageTimestamp(
          DateTime.now()
              .subtract(const Duration(days: 365))
              .millisecondsSinceEpoch,
        ),
        DateFormat('d MMM yyyy')
            .format(DateTime.now().subtract(const Duration(days: 365))),
      );
    });

    test('should return the correct name initials', () {
      expect(getInitials('Abiud Orina'), 'AO');
      expect(getInitials('Abiud Kowalski Orina'), 'AK');
    });

    test('should format username', () {
      expect(getInitials('Abiud Orina'), 'AO');
      expect(getInitials('Abiud Kowalski Orina'), 'AK');
      expect(getInitials('Abiud'), 'A');
      expect(getInitials(''), '');
    });

    test('should process membership types correctly', () async {
      expect(
        processMembership(MembershipEventTypes.invite, 'myCareHub', 'Abiud'),
        'Abiud invited myCareHub',
      );

      expect(
        processMembership(MembershipEventTypes.join, 'myCareHub', 'Abiud'),
        'myCareHub joined',
      );

      expect(
        processMembership(MembershipEventTypes.leave, 'myCareHub', 'Abiud'),
        'myCareHub left',
      );

      expect(
        processMembership(MembershipEventTypes.ban, 'myCareHub', 'Abiud'),
        'myCareHub was banned',
      );

      expect(
        processMembership(MembershipEventTypes.knock, 'myCareHub', 'Abiud'),
        'myCareHub requested to join',
      );

      expect(
        processMembership(MembershipEventTypes.unknown, 'myCareHub', 'Abiud'),
        '',
      );
    });

    test('should update room data when server data is null', () async {
      final Map<String, Room>? updatedRoomData = updateRoomData(
        fromServer: null,
        fromState: <String, Room>{
          '!testRoom:chat.savannahghi.org':
              Room.fromJson(roomMock).copyWith.call(
                    timeline: RoomTimeline.initial(),
                    state: RoomState.initial(),
                  )
        },
        invite: false,
      );

      expect(updatedRoomData?.length, 1);
      expect(updatedRoomData?.values.first.state?.events?.isEmpty, true);
    });

    test(
        'should update room data when server data is available '
        'for an existing room', () async {
      const String testSender = '@orina:chat.savannahghi.org';

      final Map<String, Room>? updatedRoomData = updateRoomData(
        fromState: SyncResponse.fromJson(syncResponseMock).rooms?.joinedRooms,
        fromServer: <String, Room>{
          '!testRoom:chat.savannahghi.org':
              Room.fromJson(roomMock).copyWith.call(
                    timeline: RoomTimeline.initial().copyWith.call(
                      // This is the value to verify

                      events: <Message>[
                        Message.fromJson(messageEventMock)
                            .copyWith
                            .call(sender: testSender),
                      ],
                    ),
                    name: 'server name',
                    topic: 'server',
                    state: RoomState.initial(),
                  )
        },
        invite: false,
      );

      expect(updatedRoomData?.values.first.timeline?.events?.length, 2);

      final Message? verifyEvent =
          updatedRoomData?.values.first.timeline?.events?.firstWhere(
        (Message? e) => e?.sender == testSender,
      );

      expect(verifyEvent, isNotNull);
      expect(verifyEvent, isA<Message>());
      expect(verifyEvent?.sender, testSender);
    });

    test(
        'should update room data when server data is available '
        'for an empty existing room', () async {
      const String testSender = '@orina:chat.savannahghi.org';

      final Map<String, Room>? updatedRoomData = updateRoomData(
        fromState: SyncResponse.fromJson(syncResponseMock).rooms?.joinedRooms,
        fromServer: <String, Room>{
          '!testRoom:chat.savannahghi.org':
              Room.fromJson(roomMock).copyWith.call(
                    name: null,
                    topic: null,
                    timeline: RoomTimeline.initial().copyWith.call(
                      // This is the value to verify

                      events: <Message>[
                        Message.fromJson(messageEventMock)
                            .copyWith
                            .call(sender: testSender),
                      ],
                    ),
                  )
        },
        invite: false,
      );

      expect(updatedRoomData?.values.first.timeline?.events?.length, 2);

      final Message? verifyEvent =
          updatedRoomData?.values.first.timeline?.events?.firstWhere(
        (Message? e) => e?.sender == testSender,
      );

      expect(verifyEvent, isNotNull);
      expect(verifyEvent, isA<Message>());
      expect(verifyEvent?.sender, testSender);
    });

    test(
        'should update room data when server data is available '
        'for a new room', () async {
      const String testSender = '@orina:chat.savannahghi.org';

      final Map<String, Room>? updatedRoomData = updateRoomData(
        fromState: SyncResponse.fromJson(syncResponseMock).rooms?.joinedRooms,
        fromServer: <String, Room>{
          '!testNewRoom:chat.savannahghi.org':
              Room.fromJson(roomMock).copyWith.call(
                    name: 'server name',
                    topic: 'server',
                    timeline: RoomTimeline.initial().copyWith.call(
                      // This is the value to verify
                      events: <Message>[
                        Message.fromJson(messageEventMock)
                            .copyWith
                            .call(sender: testSender),
                      ],
                    ),
                    state: RoomState.initial(),
                  )
        },
        invite: true,
      );

      final Room? verifyRoom = updatedRoomData?.values.firstWhere(
        (Room r) => r.roomID == '!testNewRoom:chat.savannahghi.org',
      );

      final Message? verifyEvent = verifyRoom?.timeline?.events?.firstWhere(
        (Message? e) => e?.sender == testSender,
      );

      expect(verifyEvent, isNotNull);
      expect(verifyEvent, isA<Message>());
      expect(verifyEvent?.sender, testSender);
    });

    test('should parse sync data for a joined room', () async {
      final Map<String, Room>? syncedRoomsData = enrichRooms(
        SyncResponse.fromJson(syncResponseMock).rooms?.joinedRooms,
      );

      expect(
        syncedRoomsData?.containsKey('!testRoom:chat.savannahghi.org'),
        true,
      );

      expect(
        syncedRoomsData?['!testRoom:chat.savannahghi.org']?.name,
        'The Grand Gaming Squad',
      );

      expect(
        syncedRoomsData?['!testRoom:chat.savannahghi.org']
            ?.timeline
            ?.events
            ?.isEmpty,
        false,
      );
      expect(
        syncedRoomsData?['!testRoom:chat.savannahghi.org']
            ?.state
            ?.events
            ?.isEmpty,
        false,
      );
    });

    test('should parse sync data for an invited room', () async {
      final Map<String, Room>? syncedRoomsData = enrichRooms(
        SyncResponse.fromJson(syncResponseMock).rooms?.joinedRooms,
        invite: true,
      );

      expect(
        syncedRoomsData?.containsKey('!testRoom:chat.savannahghi.org'),
        true,
      );

      expect(
        syncedRoomsData?['!testRoom:chat.savannahghi.org']?.invite,
        true,
      );
    });

    test('should parse the room name from the timeline', () async {
      final Map<String, Room>? syncedRoomsData = enrichRooms(
        SyncResponse.fromJson(syncResponseMock).rooms?.copyWith.call(
          joinedRooms: <String, Room>{
            '!testRoom:chat.savannahghi.org': Room.initial().copyWith.call(
                  timeline: RoomTimeline.initial().copyWith.call(
                    events: <Message>[
                      Message.initial()
                          .copyWith
                          .call(eventType: EventTypes.name)
                    ],
                  ),
                )
          },
        ).joinedRooms,
      );

      expect(
        syncedRoomsData?.containsKey('!testRoom:chat.savannahghi.org'),
        true,
      );

      expect(
        syncedRoomsData?['!testRoom:chat.savannahghi.org']?.invite,
        false,
      );
    });

    test('should parse the room name from the state', () async {
      final Map<String, Room>? syncedRoomsData = enrichRooms(
        SyncResponse.fromJson(syncResponseMock).rooms?.copyWith.call(
          joinedRooms: <String, Room>{
            '!testRoom:chat.savannahghi.org': Room.initial().copyWith.call(
                  state: RoomState.initial().copyWith.call(
                    events: <Message>[
                      Message.initial()
                          .copyWith
                          .call(eventType: EventTypes.avatar)
                    ],
                  ),
                )
          },
        ).joinedRooms,
      );

      expect(
        syncedRoomsData?.containsKey('!testRoom:chat.savannahghi.org'),
        true,
      );

      expect(
        syncedRoomsData?['!testRoom:chat.savannahghi.org']?.invite,
        false,
      );
    });
  });
}

==> sghi_core-0.2.44.tar.gz: test/communities/unit_tests/core/community_utils_test.dart <==
import 'dart:io';

import 'package:flutter_test/flutter_test.dart';
import 'package:intl/intl.dart';
import 'package:sghi_core/communities/core/community_utils.dart';
import 'package:sghi_core/communities/core/entities/event_types.dart';
import 'package:sghi_core/communities/core/enums.dart';
import 'package:sghi_core/communities/models/message.dart';
import 'package:sghi_core/communities/models/room.dart';
import 'package:sghi_core/communities/models/sync_response.dart';

import '../../mocks.dart';
import '../../response_mocks.dart';

void main() {
  group('utils', () {
    test('should convert mime types', () async {
      expect(() => convertMimeTypes(File('example.png'), null),
          throwsA(isA<String>()));

      expect(convertMimeTypes(File('example.png'), 'image/png'),
          equals('image/png'));

      expect(convertMimeTypes(File('example.heic'), 'image/png'),
          equals('image/heic'));
    });

    test('should pick an image', () async {
      final File? file = await selectPhoto(picker: ImagePickerMock());

      expect(file, isNotNull);
      expect(file, isA<File>());
      expect(file?.path, isNotNull);
    });

    test('should format a username', () async {
      expect(formatUsername(''), '');
      expect(formatUsername('abiud'), 'abiud');
      expect(formatUsername('abiud:chat.matrix:org'), 'abiud');
      expect(formatUsername('@abiud:chat.matrix:org'), '@abiud');
    });

    test('formatTimestamp formats timestamp correctly', () {
      const int timestamp = 1683888728201;
      final String formattedTimestamp = formatTimestamp(timestamp: timestamp);

      final String expectedFormattedTimestamp =
          '${DateFormat('d MMM, yy').format(DateTime.fromMillisecondsSinceEpoch(timestamp))} at ${DateFormat('jm').format(DateTime.fromMillisecondsSinceEpoch(timestamp))}';

      expect(formattedTimestamp, equals(expectedFormattedTimestamp));
    });

    test('formatTimestamp returns empty string when timestamp is null', () {
      final String formattedTimestamp = formatTimestamp(timestamp: null);

      expect(formattedTimestamp, equals(''));
    });

    test('should organize events by< timestamp a timestamp', () async {
      expect(sortEvents(<Message>[]), <Message>[]);

      expect(
        sortEvents(<Message>[
          Message.initial(),
          Message.initial(),
        ]),
        <Message>[
          Message.initial(),
          Message.initial(),
        ],
      );

      expect(
        sortEvents(<Message>[
          Message.initial().copyWith.call(timeStamp: 1683888728201),
          Message.initial().copyWith.call(timeStamp: 1683888728202),
        ]),
        <Message>[
          Message.initial().copyWith.call(timeStamp: 1683888728202),
          Message.initial().copyWith.call(timeStamp: 1683888728201),
        ],
      );

      expect(sortEvents(<Message>[]), <Message>[]);
    });

    test('should return a valid fibonacci sequence', () {
      expect(fibonacci(0), <int>[0]);

      expect(fibonacci(1), <int>[0, 1]);

      expect(fibonacci(2), <int>[0, 1, 1]);

      expect(fibonacci(3), <int>[0, 1, 1, 2]);
    });

    test('formatMessageTimestamp returns correct string', () {
      expect(formatMessageTimestamp(null), '');
      expect(
        formatMessageTimestamp(DateTime.now().millisecondsSinceEpoch),
        'Just now',
      );
      expect(
        formatMessageTimestamp(
          DateTime.now()
              .subtract(const Duration(seconds: 30))
              .millisecondsSinceEpoch,
        ),
        'A few seconds ago',
      );
      expect(
        formatMessageTimestamp(
          DateTime.now()
              .subtract(const Duration(minutes: 1))
              .millisecondsSinceEpoch,
        ),
        '1 minute ago',
      );
      expect(
        formatMessageTimestamp(
          DateTime.now()
              .subtract(const Duration(minutes: 5))
              .millisecondsSinceEpoch,
        ),
        '5 minutes ago',
      );
      expect(
        formatMessageTimestamp(
          DateTime.now()
              .subtract(const Duration(hours: 1))
              .millisecondsSinceEpoch,
        ),
        'Today at ${DateFormat('h:mm a').format(DateTime.now().subtract(const Duration(hours: 1)))}',
      );
      expect(
        formatMessageTimestamp(
          DateTime.now()
              .subtract(const Duration(hours: 23))
              .millisecondsSinceEpoch,
        ),
        'Yesterday at ${DateFormat('h:mm a').format(DateTime.now().subtract(const Duration(hours: 23)))}',
      );
      expect(
        formatMessageTimestamp(
          DateTime.now()
              .subtract(const Duration(days: 3))
              .millisecondsSinceEpoch,
        ),
        DateFormat('EEEE')
            .format(DateTime.now().subtract(const Duration(days: 3))),
      );
      expect(
        formatMessageTimestamp(
          DateTime.now()
              .subtract(const Duration(days: 30))
              .millisecondsSinceEpoch,
        ),
        DateFormat('d MMMM')
            .format(DateTime.now().subtract(const Duration(days: 30))),
      );
      expect(
        formatMessageTimestamp(
          DateTime.now()
              .subtract(const Duration(days: 365))
              .millisecondsSinceEpoch,
        ),
        DateFormat('d MMM yyyy')
            .format(DateTime.now().subtract(const Duration(days: 365))),
      );
    });

    test('should return the correct name initials', () {
      expect(getInitials('Abiud Orina'), 'AO');
      expect(getInitials('Abiud Kowalski Orina'), 'AK');
    });

    test('should format username', () {
      expect(getInitials('Abiud Orina'), 'AO');
      expect(getInitials('Abiud Kowalski Orina'), 'AK');
      expect(getInitials('Abiud'), 'A');
      expect(getInitials(''), '');
    });

    test('should process membership types correctly', () async {
      expect(
        processMembership(
          sender: 'Abiud',
          target: 'myCareHub',
          type: MembershipEventTypes.invite,
        ),
        'Abiud invited myCareHub to the conversation',
      );

      expect(
        processMembership(
          sender: 'Abiud',
          target: '',
          type: MembershipEventTypes.join,
        ),
        'Abiud joined the conversation',
      );

      expect(
        processMembership(
          sender: 'Abiud',
          target: '',
          type: MembershipEventTypes.leave,
        ),
        'Abiud left the conversation',
      );

      expect(
        processMembership(
          sender: 'Abiud',
          target: 'testUser',
          type: MembershipEventTypes.ban,
        ),
        'testUser was banned from the conversation by Abiud',
      );
    });

    test('should update room data when server data is null', () async {
      final Map<String, Room>? updatedRoomData = updateRoomData(
        fromServer: null,
        fromState: <String, Room>{
          '!testRoom:chat.savannahghi.org':
              Room.fromJson(roomMock).copyWith.call(
                    timeline: RoomTimeline.initial(),
                    state: RoomState.initial(),
                  )
        },
        invite: false,
      );

      expect(updatedRoomData?.length, 1);
      expect(updatedRoomData?.values.first.state?.events?.isEmpty, true);
    });

    test(
        'should update room data when server data is available '
        'for an existing room', () async {
      const String testSender = '@orina:chat.savannahghi.org';

      final Map<String, Room>? updatedRoomData = updateRoomData(
        fromState: SyncResponse.fromJson(syncResponseMock).rooms?.joinedRooms,
        fromServer: <String, Room>{
          '!testRoom:chat.savannahghi.org':
              Room.fromJson(roomMock).copyWith.call(
                    timeline: RoomTimeline.initial().copyWith.call(
                      // This is the value to verify

                      events: <Message>[
                        Message.fromJson(messageEventMock)
                            .copyWith
                            .call(sender: testSender),
                      ],
                    ),
                    name: 'server name',
                    topic: 'server',
                    state: RoomState.initial(),
                  )
        },
        invite: false,
      );

      expect(updatedRoomData?.values.first.timeline?.events?.length, 2);

      final Message? verifyEvent =
          updatedRoomData?.values.first.timeline?.events?.firstWhere(
        (Message? e) => e?.sender == testSender,
      );

      expect(verifyEvent, isNotNull);
      expect(verifyEvent, isA<Message>());
      expect(verifyEvent?.sender, testSender);
    });

    test(
        'should update room data when server data is available '
        'for an empty existing room', () async {
      const String testSender = '@orina:chat.savannahghi.org';

      final Map<String, Room>? updatedRoomData = updateRoomData(
        fromState: SyncResponse.fromJson(syncResponseMock).rooms?.joinedRooms,
        fromServer: <String, Room>{
          '!testRoom:chat.savannahghi.org':
              Room.fromJson(roomMock).copyWith.call(
                    name: null,
                    topic: null,
                    timeline: RoomTimeline.initial().copyWith.call(
                      // This is the value to verify

                      events: <Message>[
                        Message.fromJson(messageEventMock)
                            .copyWith
                            .call(sender: testSender),
                      ],
                    ),
                  )
        },
        invite: false,
      );

      expect(updatedRoomData?.values.first.timeline?.events?.length, 2);

      final Message? verifyEvent =
          updatedRoomData?.values.first.timeline?.events?.firstWhere(
        (Message? e) => e?.sender == testSender,
      );

      expect(verifyEvent, isNotNull);
      expect(verifyEvent, isA<Message>());
      expect(verifyEvent?.sender, testSender);
    });

    test(
        'should update room data when server data is available '
        'for a new room', () async {
      const String testSender = '@orina:chat.savannahghi.org';

      final Map<String, Room>? updatedRoomData = updateRoomData(
        fromState: SyncResponse.fromJson(syncResponseMock).rooms?.joinedRooms,
        fromServer: <String, Room>{
          '!testNewRoom:chat.savannahghi.org':
              Room.fromJson(roomMock).copyWith.call(
                    name: 'server name',
                    topic: 'server',
                    timeline: RoomTimeline.initial().copyWith.call(
                      // This is the value to verify
                      events: <Message>[
                        Message.fromJson(messageEventMock)
                            .copyWith
                            .call(sender: testSender),
                      ],
                    ),
                    state: RoomState.initial(),
                  )
        },
        invite: true,
      );

      final Room? verifyRoom = updatedRoomData?.values.firstWhere(
        (Room r) => r.roomID == '!testNewRoom:chat.savannahghi.org',
      );

      final Message? verifyEvent = verifyRoom?.timeline?.events?.firstWhere(
        (Message? e) => e?.sender == testSender,
      );

      expect(verifyEvent, isNotNull);
      expect(verifyEvent, isA<Message>());
      expect(verifyEvent?.sender, testSender);
    });

    test('should parse sync data for a joined room', () async {
      final Map<String, Room>? syncedRoomsData = enrichRooms(
        SyncResponse.fromJson(syncResponseMock).rooms?.joinedRooms,
      );

      expect(
        syncedRoomsData?.containsKey('!testRoom:chat.savannahghi.org'),
        true,
      );

      expect(
        syncedRoomsData?['!testRoom:chat.savannahghi.org']?.name,
        'The Grand Gaming Squad',
      );

      expect(
        syncedRoomsData?['!testRoom:chat.savannahghi.org']
            ?.timeline
            ?.events
            ?.isEmpty,
        false,
      );
      expect(
        syncedRoomsData?['!testRoom:chat.savannahghi.org']
            ?.state
            ?.events
            ?.isEmpty,
        false,
      );
    });

    test('should parse sync data for an invited room', () async {
      final Map<String, Room>? syncedRoomsData = enrichRooms(
        SyncResponse.fromJson(syncResponseMock).rooms?.joinedRooms,
        invite: true,
      );

      expect(
        syncedRoomsData?.containsKey('!testRoom:chat.savannahghi.org'),
        true,
      );

      expect(
        syncedRoomsData?['!testRoom:chat.savannahghi.org']?.invite,
        true,
      );
    });

    test('should parse the room name from the timeline', () async {
      final Map<String, Room>? syncedRoomsData = enrichRooms(
        SyncResponse.fromJson(syncResponseMock).rooms?.copyWith.call(
          joinedRooms: <String, Room>{
            '!testRoom:chat.savannahghi.org': Room.initial().copyWith.call(
                  timeline: RoomTimeline.initial().copyWith.call(
                    events: <Message>[
                      Message.initial()
                          .copyWith
                          .call(eventType: EventTypes.name)
                    ],
                  ),
                )
          },
        ).joinedRooms,
      );

      expect(
        syncedRoomsData?.containsKey('!testRoom:chat.savannahghi.org'),
        true,
      );

      expect(
        syncedRoomsData?['!testRoom:chat.savannahghi.org']?.invite,
        false,
      );
    });

    test('should parse the room name from the state', () async {
      final Map<String, Room>? syncedRoomsData = enrichRooms(
        SyncResponse.fromJson(syncResponseMock).rooms?.copyWith.call(
          joinedRooms: <String, Room>{
            '!testRoom:chat.savannahghi.org': Room.initial().copyWith.call(
                  state: RoomState.initial().copyWith.call(
                    events: <Message>[
                      Message.initial()
                          .copyWith
                          .call(eventType: EventTypes.avatar)
                    ],
                  ),
                )
          },
        ).joinedRooms,
      );

      expect(
        syncedRoomsData?.containsKey('!testRoom:chat.savannahghi.org'),
        true,
      );

      expect(
        syncedRoomsData?['!testRoom:chat.savannahghi.org']?.invite,
        false,
      );
    });

    group('processRoomUpdates', () {
      test('should remove duplicate events and redacted events', () {
        final Room serverRoom = Room(
          state: RoomState(events: <Message>[
            Message(eventType: 'message', eventID: '1', timeStamp: 1),
            // Intentional duplicate
            Message(eventType: 'message', eventID: '1', timeStamp: 1),
          ]),
          timeline: RoomTimeline(events: <Message>[
            Message(eventType: 'message', eventID: '2', timeStamp: 2),
            Message(eventType: 'message', eventID: '3', timeStamp: 3),
            Message(
              eventType: 'm.room.redaction',
              redacts: '3',
              eventID: '4',
              timeStamp: 4,
            ),
          ]),
        );

        final Room stateRoom = Room(
          state: RoomState(events: <Message>[
            Message(eventType: 'message', eventID: '4', timeStamp: 4),
            Message(
                eventType: 'm.room.redaction',
                redacts: '5',
                eventID: '6',
                timeStamp: 6),
          ]),
          timeline: RoomTimeline(events: <Message>[
            Message(eventType: 'message', eventID: '7', timeStamp: 7),
          ]),
        );

        final Room processedRoom = processRoomUpdates(
          serverRoom: serverRoom,
          stateRoom: stateRoom,
          invite: false,
        );

        expect(processedRoom.state?.events?.length, 2);
        expect(processedRoom.state?.events?[0]?.eventID, '4');
        expect(processedRoom.state?.events?[1]?.eventID, '1');

        expect(processedRoom.timeline?.events?.length, 3);
        expect(processedRoom.timeline?.events?[0]?.eventID, '7');
        expect(processedRoom.timeline?.events?[1]?.eventID, '3');
      });

      test('should preserve other properties of the room', () {
        final Room serverRoom = Room(
          name: 'Room Name',
          topic: 'Room Topic',
        );

        final Room stateRoom = Room();

        final Room processedRoom = processRoomUpdates(
          serverRoom: serverRoom,
          stateRoom: stateRoom,
          invite: true,
        );

        expect(processedRoom.name, 'Room Name');
        expect(processedRoom.topic, 'Room Topic');
        expect(processedRoom.invite, true);
      });

      test(
          'processRoomUpdates handles adding new message and redacting'
          ' existing message', () {
        // Create initial state room and server room
        final Room stateRoom = Room(
          timeline: RoomTimeline(events: <Message>[
            Message(
              eventID: '1',
              eventType: EventTypes.message,
              content: MessageContent(body: 'Hello'),
            ),
            Message(
              eventID: '2',
              eventType: EventTypes.message,
              content: MessageContent(body: 'World'),
            ),
          ]),
        );

        final Room serverRoom = Room(
          timeline: RoomTimeline(events: <Message>[
            Message(
              eventID: '3',
              eventType: EventTypes.message,
              content: MessageContent(body: 'Hello World'),
            ),
            Message(
              eventType: 'm.room.redaction',
              sender: 'admin@prohealth360.org',
              content: MessageContent(),
              timeStamp: DateTime.now().millisecondsSinceEpoch,
              eventID: '4',
              redacts: '2',
              roomID: 'room_id',
              stateKey: '1',
            ),
          ]),
        );

        // Perform room updates
        final Room processedRoom = processRoomUpdates(
          serverRoom: serverRoom,
          stateRoom: stateRoom,
          invite: false,
        );

        /// There should be 2 events since the redactions have been applied and
        /// removed
        expect(processedRoom.timeline?.events?.length, 3);
        expect(processedRoom.timeline?.events?.first?.eventID, '1');
        expect(processedRoom.timeline?.events?[1]?.eventID, '2');
      });
    });
  });
}
